import _, { round } from 'lodash';
import react, { useState, useEffect } from 'react';

import GPXOverlay from './gpx-overlay';
import Tile from './tile';

import '../theme/map.css';

export const tileSize = 256;

// cf https://wiki.openstreetmap.org/wiki/Slippy_map_tilenames#ECMAScript_(JavaScript/ActionScript,_etc.)

const lon2tile = (lon: number, zoom: number) => {
  const real = ((lon + 180) / 360) * Math.pow(2, zoom);
  const floor = Math.floor(real);
  return [floor, real - floor];
};
const lat2tile = (lat: number, zoom: number) => {
  const real =
    ((1 -
      Math.log(
        Math.tan((lat * Math.PI) / 180) + 1 / Math.cos((lat * Math.PI) / 180)
      ) /
        Math.PI) /
      2) *
    Math.pow(2, zoom);
    const floor = Math.floor(real);
    return [floor, real - floor];
  };

const Map: react.FC = () => {
  const initialCenter: [number, number] = [43.57029965, 3.94242897];
  const initialZoom: number = 15;
  const [center, setCenter] = useState(initialCenter);
  const [zoom, setZoom] = useState(initialZoom);

  const [dimensions, setDimensions] = useState({
    height: window.innerHeight,
    width: window.innerWidth,
  });

  useEffect(() => {
    const handleResize = () => {
      setDimensions({
        height: window.innerHeight,
        width: window.innerWidth,
      });
    };
    window.addEventListener('resize', handleResize);
  });

  const nbTilesY = _.ceil(dimensions.height / tileSize) + 2;
  const nbTilesX = _.ceil(dimensions.width / tileSize) + 2;
  const [tileCenterY, deltaY] = lat2tile(center[0], zoom);
  const [tileCenterX, deltaX] = lon2tile(center[1], zoom);
  const firstTileY = tileCenterY - _.ceil(nbTilesY / 2);
  const firstTileX = tileCenterX - _.ceil(nbTilesX / 2);
  


  return (
    <div className='tiles'>
      {_.range(nbTilesY).map((iy) => (
        <div key={'y' + iy} className='tilesRow'>
          {_.range(nbTilesX).map((ix) => (
            <Tile
              key={'x' + ix}
              iy={iy}
              ix={ix}
              x={firstTileX + ix}
              y={firstTileY + iy}
              deltaX ={deltaX}
              deltaY ={deltaY}
              zoom={zoom}
            />
          ))}
        </div>
      ))}
    </div>
  );
};

export default Map;
